<?php

namespace FiloBlu\OrderStatus\Observer;

use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Magento\Store\Model\ScopeInterface;

/**
 * Observer on Adyen notification and set order status accordingly
 */
 
 class UpdateStatusOnInvoice implements ObserverInterface {
    
    /**
     * @var \Psr\Log\LoggerInterface $logger
     */  
    protected $_logger;
    
    protected $_config;
    
    const WHITELISTED_METHODS = array('msp_cashondelivery', 'amazon_payment');
    const AUTHORIZED_ORDER_STATUS = 'confirmed';
    const AUTHORIZED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_pre_authorized';
    const CAPTURED_ORDER_STATUS = 'payment_confirmed';
    const CAPTURED_ORDER_STATUS_CONFIG_PATH = 'payment/adyen_abstract/payment_authorized';
    const DEBUG_LOG_ACTIVE = TRUE;
	
    /**
     * @param \Psr\Log\LoggerInterface $logger
     */  
	public function __construct(
		\Psr\Log\LoggerInterface $logger,
                ScopeConfigInterface $config
	){
		$this->_logger = $logger;
                $this->_config = $config;
	}

    /**
     * Process order
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer){
        /* @var $order \Magento\Sales\Model\Order */
        $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Processing Invoice creation");

        $event = $observer->getEvent();
        $invoice = $event['invoice'];
        $order = $invoice->getOrder();
        $payment_method = $order->getPayment()->getMethod();
        if (self::DEBUG_LOG_ACTIVE) {
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Processing Order");
            $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: payment_method: $payment_method");
        }
        if (in_array($payment_method, self::WHITELISTED_METHODS)) {
            $captured_order_status = $this->_config->getValue(self::CAPTURED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) ? 
                $this->_config->getValue(self::CAPTURED_ORDER_STATUS_CONFIG_PATH, ScopeInterface::SCOPE_STORE) : self::CAPTURED_ORDER_STATUS;
            $order->addStatusHistoryComment(
                __('Order processed by mod2-order-status'), $captured_order_status);
            if (self::DEBUG_LOG_ACTIVE) {
                $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Order status set to " . $captured_order_status);
            }
        }
        return TRUE;
    }
}