<?php

namespace FiloBlu\OrderStatus\Observer;

use FiloBlu\OrderStatus\Helper\Data;
use \Magento\Framework\Event\ObserverInterface;
use \Magento\Framework\Event\Observer;
use \Magento\Framework\App\Config\ScopeConfigInterface;
use \Magento\Store\Model\ScopeInterface;

class UpdateStatusVerify implements ObserverInterface
{
    const DEBUG_LOG_ACTIVE = TRUE;

    /**
     * @var \Psr\Log\LoggerInterface $_logger
     */
    protected $_logger;

    /**
     * @var Data $helper
     */
    protected $helper;

    /**
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Psr\Log\LoggerInterface $logger,
        ScopeConfigInterface $config,
        Data $helper
    ){
        $this->_logger = $logger;
        $this->_config = $config;
        $this->helper = $helper;
    }

    /**
     * Process order
     *
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(Observer $observer) {

        $order = $observer->getEvent()->getOrder();
        if($order->getForceUpdateStatus()){
            return $order;
        }
        $fromStatus = $order->getOrigData("status");
        $toStatus=$order->getData("status");
        $hasInvoices = $order->hasInvoices();

        if(!$this->helper->isValidStatusUpdate($fromStatus, $toStatus, $hasInvoices))
        {
            if (self::DEBUG_LOG_ACTIVE)
            {
                $this->_logger->addInfo("FiloBlu\OrderStatus\Observer: Status cannot be updated in payment_confirmed without invoice, order ".$order->getIncrementId());
            }
            throw new \Magento\Framework\Exception\LocalizedException(__("Status update not allowed!"));
        }

        return $order;
    }
}